/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.common.model.bulletin;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.common.model.WithMetadata;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link LeveledMessage}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new LeveledMessage.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableLeveledMessage.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "LeveledMessage"})
final class ImmutableLeveledMessage implements LeveledMessage {
  private final LeveledMessage.Level level;
  private final LeveledMessage.Code code;
  private final String message;
  private final Map<String, String> metadata;

  private ImmutableLeveledMessage(
      LeveledMessage.Level level,
      LeveledMessage.Code code,
      Optional<String> message,
      Map<String, ? extends String> metadata) {
    this.level = Objects.requireNonNull(level, "level");
    this.code = Objects.requireNonNull(code, "code");
    this.message = message.orElse(null);
    this.metadata = createUnmodifiableMap(true, false, metadata);
    initShim.level(this.level);
    initShim.code(this.code);
    initShim.metadata(this.metadata);
    this.initShim = null;
  }

  private ImmutableLeveledMessage(ImmutableLeveledMessage.Builder builder) {
    this.message = builder.message;
    if (builder.level != null) {
      initShim.level(builder.level);
    }
    if (builder.code != null) {
      initShim.code(builder.code);
    }
    if (builder.metadataIsSet()) {
      initShim.metadata(createUnmodifiableMap(false, false, builder.metadata));
    }
    this.level = initShim.getLevel();
    this.code = initShim.getCode();
    this.metadata = initShim.getMetadata();
    this.initShim = null;
  }

  private ImmutableLeveledMessage(
      ImmutableLeveledMessage original,
      LeveledMessage.Level level,
      LeveledMessage.Code code,
      String message,
      Map<String, String> metadata) {
    this.level = level;
    this.code = code;
    this.message = message;
    this.metadata = metadata;
    this.initShim = null;
  }

  private static final int STAGE_INITIALIZING = -1;
  private static final int STAGE_UNINITIALIZED = 0;
  private static final int STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  private final class InitShim {
    private LeveledMessage.Level level;
    private int levelBuildStage;

    LeveledMessage.Level getLevel() {
      if (levelBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (levelBuildStage == STAGE_UNINITIALIZED) {
        levelBuildStage = STAGE_INITIALIZING;
        this.level = Objects.requireNonNull(getLevelInitialize(), "level");
        levelBuildStage = STAGE_INITIALIZED;
      }
      return this.level;
    }

    void level(LeveledMessage.Level level) {
      this.level = level;
      levelBuildStage = STAGE_INITIALIZED;
    }
    private LeveledMessage.Code code;
    private int codeBuildStage;

    LeveledMessage.Code getCode() {
      if (codeBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (codeBuildStage == STAGE_UNINITIALIZED) {
        codeBuildStage = STAGE_INITIALIZING;
        this.code = Objects.requireNonNull(getCodeInitialize(), "code");
        codeBuildStage = STAGE_INITIALIZED;
      }
      return this.code;
    }

    void code(LeveledMessage.Code code) {
      this.code = code;
      codeBuildStage = STAGE_INITIALIZED;
    }
    private Map<String, String> metadata;
    private int metadataBuildStage;

    Map<String, String> getMetadata() {
      if (metadataBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (metadataBuildStage == STAGE_UNINITIALIZED) {
        metadataBuildStage = STAGE_INITIALIZING;
        this.metadata = createUnmodifiableMap(true, false, getMetadataInitialize());
        metadataBuildStage = STAGE_INITIALIZED;
      }
      return this.metadata;
    }

    void metadata(Map<String, String> metadata) {
      this.metadata = metadata;
      metadataBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      ArrayList<String> attributes = new ArrayList<String>();
      if (levelBuildStage == STAGE_INITIALIZING) attributes.add("level");
      if (codeBuildStage == STAGE_INITIALIZING) attributes.add("code");
      if (metadataBuildStage == STAGE_INITIALIZING) attributes.add("metadata");
      return "Cannot build LeveledMessage, attribute initializers form cycle" + attributes;
    }
  }

  private LeveledMessage.Level getLevelInitialize() {
    return LeveledMessage.super.getLevel();
  }

  private LeveledMessage.Code getCodeInitialize() {
    return LeveledMessage.super.getCode();
  }

  private Map<String, String> getMetadataInitialize() {
    return LeveledMessage.super.getMetadata();
  }

  /**
   * @return The value of the {@code level} attribute
   */
  @JsonProperty("level")
  @Override
  public LeveledMessage.Level getLevel() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getLevel()
        : this.level;
  }

  /**
   * @return The value of the {@code code} attribute
   */
  @JsonProperty("code")
  @Override
  public LeveledMessage.Code getCode() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getCode()
        : this.code;
  }

  /**
   * @return The value of the {@code message} attribute
   */
  @JsonProperty("message")
  @Override
  public Optional<String> getMessage() {
    return Optional.ofNullable(message);
  }

  /**
   * @return The value of the {@code metadata} attribute
   */
  @JsonProperty("metadata")
  @Override
  public Map<String, String> getMetadata() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getMetadata()
        : this.metadata;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link LeveledMessage#getLevel() level} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for level
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableLeveledMessage withLevel(LeveledMessage.Level value) {
    if (this.level == value) return this;
    LeveledMessage.Level newValue = Objects.requireNonNull(value, "level");
    return validate(new ImmutableLeveledMessage(this, newValue, this.code, this.message, this.metadata));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link LeveledMessage#getCode() code} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for code
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableLeveledMessage withCode(LeveledMessage.Code value) {
    if (this.code == value) return this;
    LeveledMessage.Code newValue = Objects.requireNonNull(value, "code");
    return validate(new ImmutableLeveledMessage(this, this.level, newValue, this.message, this.metadata));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link LeveledMessage#getMessage() message} attribute.
   * @param value The value for message
   * @return A modified copy of {@code this} object
   */
  public final ImmutableLeveledMessage withMessage(String value) {
    String newValue = Objects.requireNonNull(value, "message");
    if (Objects.equals(this.message, newValue)) return this;
    return validate(new ImmutableLeveledMessage(this, this.level, this.code, newValue, this.metadata));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link LeveledMessage#getMessage() message} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for message
   * @return A modified copy of {@code this} object
   */
  public final ImmutableLeveledMessage withMessage(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.message, value)) return this;
    return validate(new ImmutableLeveledMessage(this, this.level, this.code, value, this.metadata));
  }

  /**
   * Copy the current immutable object by replacing the {@link LeveledMessage#getMetadata() metadata} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the metadata map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableLeveledMessage withMetadata(Map<String, ? extends String> entries) {
    if (this.metadata == entries) return this;
    Map<String, String> newValue = createUnmodifiableMap(true, false, entries);
    return validate(new ImmutableLeveledMessage(this, this.level, this.code, this.message, newValue));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableLeveledMessage} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableLeveledMessage
        && equalTo((ImmutableLeveledMessage) another);
  }

  private boolean equalTo(ImmutableLeveledMessage another) {
    return level.equals(another.level)
        && code.equals(another.code)
        && Objects.equals(message, another.message)
        && metadata.equals(another.metadata);
  }

  /**
   * Computes a hash code from attributes: {@code level}, {@code code}, {@code message}, {@code metadata}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + level.hashCode();
    h += (h << 5) + code.hashCode();
    h += (h << 5) + Objects.hashCode(message);
    h += (h << 5) + metadata.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code LeveledMessage} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("LeveledMessage{");
    builder.append("level=").append(level);
    builder.append(", ");
    builder.append("code=").append(code);
    if (message != null) {
      builder.append(", ");
      builder.append("message=").append(message);
    }
    builder.append(", ");
    builder.append("metadata=").append(metadata);
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code LeveledMessage} instance.
   * @param level The value for the {@code level} attribute
   * @param code The value for the {@code code} attribute
   * @param message The value for the {@code message} attribute
   * @param metadata The value for the {@code metadata} attribute
   * @return An immutable LeveledMessage instance
   */
  public static LeveledMessage of(LeveledMessage.Level level, LeveledMessage.Code code, Optional<String> message, Map<String, ? extends String> metadata) {
    return validate(new ImmutableLeveledMessage(level, code, message, metadata));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableLeveledMessage validate(ImmutableLeveledMessage instance) {
    Set<ConstraintViolation<ImmutableLeveledMessage>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link LeveledMessage} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable LeveledMessage instance
   */
  public static LeveledMessage copyOf(LeveledMessage instance) {
    if (instance instanceof ImmutableLeveledMessage) {
      return (ImmutableLeveledMessage) instance;
    }
    return new LeveledMessage.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link LeveledMessage LeveledMessage}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  public static class Builder {
    private static final long OPT_BIT_METADATA = 0x1L;
    private long optBits;

    private LeveledMessage.Level level;
    private LeveledMessage.Code code;
    private String message;
    private Map<String, String> metadata = new LinkedHashMap<String, String>();

    /**
     * Creates a builder for {@link LeveledMessage LeveledMessage} instances.
     */
    public Builder() {
      if (!(this instanceof LeveledMessage.Builder)) {
        throw new UnsupportedOperationException("Use: new LeveledMessage.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.bulletin.LeveledMessage} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final LeveledMessage.Builder createFrom(LeveledMessage instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (LeveledMessage.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithMetadata} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final LeveledMessage.Builder createFrom(WithMetadata instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (LeveledMessage.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof LeveledMessage) {
        LeveledMessage instance = (LeveledMessage) object;
        Optional<String> messageOptional = instance.getMessage();
        if (messageOptional.isPresent()) {
          message(messageOptional);
        }
        level(instance.getLevel());
        code(instance.getCode());
      }
      if (object instanceof WithMetadata) {
        WithMetadata instance = (WithMetadata) object;
        putAllMetadata(instance.getMetadata());
      }
    }

    /**
     * Initializes the value for the {@link LeveledMessage#getLevel() level} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link LeveledMessage#getLevel() level}.</em>
     * @param level The value for level 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("level")
    public final LeveledMessage.Builder level(LeveledMessage.Level level) {
      this.level = Objects.requireNonNull(level, "level");
      return (LeveledMessage.Builder) this;
    }

    /**
     * Initializes the value for the {@link LeveledMessage#getCode() code} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link LeveledMessage#getCode() code}.</em>
     * @param code The value for code 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("code")
    public final LeveledMessage.Builder code(LeveledMessage.Code code) {
      this.code = Objects.requireNonNull(code, "code");
      return (LeveledMessage.Builder) this;
    }

    /**
     * Initializes the optional value {@link LeveledMessage#getMessage() message} to message.
     * @param message The value for message
     * @return {@code this} builder for chained invocation
     */
    public final LeveledMessage.Builder message(String message) {
      this.message = Objects.requireNonNull(message, "message");
      return (LeveledMessage.Builder) this;
    }

    /**
     * Initializes the optional value {@link LeveledMessage#getMessage() message} to message.
     * @param message The value for message
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("message")
    public final LeveledMessage.Builder message(Optional<String> message) {
      this.message = message.orElse(null);
      return (LeveledMessage.Builder) this;
    }

    /**
     * Put one entry to the {@link LeveledMessage#getMetadata() metadata} map.
     * @param key The key in the metadata map
     * @param value The associated value in the metadata map
     * @return {@code this} builder for use in a chained invocation
     */
    public final LeveledMessage.Builder putMetadata(String key, String value) {
      this.metadata.put(
          Objects.requireNonNull(key, "metadata key"),
          Objects.requireNonNull(value, "metadata value"));
      optBits |= OPT_BIT_METADATA;
      return (LeveledMessage.Builder) this;
    }

    /**
     * Put one entry to the {@link LeveledMessage#getMetadata() metadata} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final LeveledMessage.Builder putMetadata(Map.Entry<String, ? extends String> entry) {
      String k = entry.getKey();
      String v = entry.getValue();
      this.metadata.put(
          Objects.requireNonNull(k, "metadata key"),
          Objects.requireNonNull(v, "metadata value"));
      optBits |= OPT_BIT_METADATA;
      return (LeveledMessage.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link LeveledMessage#getMetadata() metadata} map. Nulls are not permitted
     * @param metadata The entries that will be added to the metadata map
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("metadata")
    public final LeveledMessage.Builder metadata(Map<String, ? extends String> metadata) {
      this.metadata.clear();
      optBits |= OPT_BIT_METADATA;
      return putAllMetadata(metadata);
    }

    /**
     * Put all mappings from the specified map as entries to {@link LeveledMessage#getMetadata() metadata} map. Nulls are not permitted
     * @param metadata The entries that will be added to the metadata map
     * @return {@code this} builder for use in a chained invocation
     */
    public final LeveledMessage.Builder putAllMetadata(Map<String, ? extends String> metadata) {
      for (Map.Entry<String, ? extends String> entry : metadata.entrySet()) {
        String k = entry.getKey();
        String v = entry.getValue();
        this.metadata.put(
            Objects.requireNonNull(k, "metadata key"),
            Objects.requireNonNull(v, "metadata value"));
      }
      optBits |= OPT_BIT_METADATA;
      return (LeveledMessage.Builder) this;
    }

    /**
     * Builds a new {@link LeveledMessage LeveledMessage}.
     * @return An immutable instance of LeveledMessage
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public LeveledMessage build() {
      return ImmutableLeveledMessage.validate(new ImmutableLeveledMessage(this));
    }

    private boolean metadataIsSet() {
      return (optBits & OPT_BIT_METADATA) != 0;
    }
  }

  private static <K, V> Map<K, V> createUnmodifiableMap(boolean checkNulls, boolean skipNulls, Map<? extends K, ? extends V> map) {
    switch (map.size()) {
    case 0: return Collections.emptyMap();
    case 1: {
      Map.Entry<? extends K, ? extends V> e = map.entrySet().iterator().next();
      K k = e.getKey();
      V v = e.getValue();
      if (checkNulls) {
        Objects.requireNonNull(k, "key");
        Objects.requireNonNull(v, "value");
      }
      if (skipNulls && (k == null || v == null)) {
        return Collections.emptyMap();
      }
      return Collections.singletonMap(k, v);
    }
    default: {
      Map<K, V> linkedMap = new LinkedHashMap<K, V>(map.size());
      if (skipNulls || checkNulls) {
        for (Map.Entry<? extends K, ? extends V> e : map.entrySet()) {
          K k = e.getKey();
          V v = e.getValue();
          if (skipNulls) {
            if (k == null || v == null) continue;
          } else if (checkNulls) {
            Objects.requireNonNull(k, "key");
            Objects.requireNonNull(v, "value");
          }
          linkedMap.put(k, v);
        }
      } else {
        linkedMap.putAll(map);
      }
      return Collections.unmodifiableMap(linkedMap);
    }
    }
  }
}
