/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.common.model.integration;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.common.model.ResourceIdentifier;
import io.syndesis.common.model.WithModificationTimestamps;
import io.syndesis.common.model.WithName;
import io.syndesis.common.model.WithResourceId;
import io.syndesis.common.model.WithTags;
import io.syndesis.common.model.WithVersion;
import io.syndesis.common.model.connection.Connection;
import io.syndesis.common.util.json.OptionalStringTrimmingConverter;
import io.syndesis.common.util.json.StringTrimmingConverter;
import java.io.ObjectStreamException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.NavigableSet;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link Integration}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new Integration.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableIntegration.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "Integration"})
final class ImmutableIntegration implements Integration {
  private final String id;
  private final boolean isDeleted;
  private final List<Connection> connections;
  private final List<Step> steps;
  private final List<ResourceIdentifier> resources;
  private final Scheduler scheduler;
  private final String description;
  private final int version;
  private final long createdAt;
  private final long updatedAt;
  private final SortedSet<String> tags;
  private final String name;

  private ImmutableIntegration(
      Optional<String> id,
      boolean isDeleted,
      Iterable<? extends Connection> connections,
      Iterable<? extends Step> steps,
      Iterable<? extends ResourceIdentifier> resources,
      Optional<Scheduler> scheduler,
      Optional<String> description,
      int version,
      long createdAt,
      long updatedAt,
      Iterable<String> tags,
      String name) {
    this.id = id.orElse(null);
    this.isDeleted = isDeleted;
    this.connections = createUnmodifiableList(false, createSafeList(connections, true, false));
    this.steps = createUnmodifiableList(false, createSafeList(steps, true, false));
    this.resources = createUnmodifiableList(false, createSafeList(resources, true, false));
    this.scheduler = scheduler.orElse(null);
    this.description = description.orElse(null);
    this.version = version;
    this.createdAt = createdAt;
    this.updatedAt = updatedAt;
    this.tags = createUnmodifiableSortedSet(false, createSafeList(tags, false, true));
    this.name = name;
    initShim.isDeleted(this.isDeleted);
    initShim.connections(this.connections);
    initShim.steps(this.steps);
    initShim.resources(this.resources);
    initShim.version(this.version);
    initShim.createdAt(this.createdAt);
    initShim.updatedAt(this.updatedAt);
    this.initShim = null;
  }

  private ImmutableIntegration(ImmutableIntegration.Builder builder) {
    this.id = builder.id;
    this.scheduler = builder.scheduler;
    this.description = builder.description;
    this.tags = createUnmodifiableSortedSet(false, createSafeList(builder.tags, false, false));
    this.name = builder.name;
    if (builder.isDeletedIsSet()) {
      initShim.isDeleted(builder.isDeleted);
    }
    if (builder.connectionsIsSet()) {
      initShim.connections(createUnmodifiableList(true, builder.connections));
    }
    if (builder.stepsIsSet()) {
      initShim.steps(createUnmodifiableList(true, builder.steps));
    }
    if (builder.resourcesIsSet()) {
      initShim.resources(createUnmodifiableList(true, builder.resources));
    }
    if (builder.versionIsSet()) {
      initShim.version(builder.version);
    }
    if (builder.createdAtIsSet()) {
      initShim.createdAt(builder.createdAt);
    }
    if (builder.updatedAtIsSet()) {
      initShim.updatedAt(builder.updatedAt);
    }
    this.isDeleted = initShim.isDeleted();
    this.connections = initShim.getConnections();
    this.steps = initShim.getSteps();
    this.resources = initShim.getResources();
    this.version = initShim.getVersion();
    this.createdAt = initShim.getCreatedAt();
    this.updatedAt = initShim.getUpdatedAt();
    this.initShim = null;
  }

  private ImmutableIntegration(
      ImmutableIntegration original,
      String id,
      boolean isDeleted,
      List<Connection> connections,
      List<Step> steps,
      List<ResourceIdentifier> resources,
      Scheduler scheduler,
      String description,
      int version,
      long createdAt,
      long updatedAt,
      SortedSet<String> tags,
      String name) {
    this.id = id;
    this.isDeleted = isDeleted;
    this.connections = connections;
    this.steps = steps;
    this.resources = resources;
    this.scheduler = scheduler;
    this.description = description;
    this.version = version;
    this.createdAt = createdAt;
    this.updatedAt = updatedAt;
    this.tags = tags;
    this.name = name;
    this.initShim = null;
  }

  private static final int STAGE_INITIALIZING = -1;
  private static final int STAGE_UNINITIALIZED = 0;
  private static final int STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  private final class InitShim {
    private boolean isDeleted;
    private int isDeletedBuildStage;

    boolean isDeleted() {
      if (isDeletedBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (isDeletedBuildStage == STAGE_UNINITIALIZED) {
        isDeletedBuildStage = STAGE_INITIALIZING;
        this.isDeleted = isDeletedInitialize();
        isDeletedBuildStage = STAGE_INITIALIZED;
      }
      return this.isDeleted;
    }

    void isDeleted(boolean isDeleted) {
      this.isDeleted = isDeleted;
      isDeletedBuildStage = STAGE_INITIALIZED;
    }
    private List<Connection> connections;
    private int connectionsBuildStage;

    List<Connection> getConnections() {
      if (connectionsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (connectionsBuildStage == STAGE_UNINITIALIZED) {
        connectionsBuildStage = STAGE_INITIALIZING;
        this.connections = createUnmodifiableList(false, createSafeList(getConnectionsInitialize(), true, false));
        connectionsBuildStage = STAGE_INITIALIZED;
      }
      return this.connections;
    }

    void connections(List<Connection> connections) {
      this.connections = connections;
      connectionsBuildStage = STAGE_INITIALIZED;
    }
    private List<Step> steps;
    private int stepsBuildStage;

    List<Step> getSteps() {
      if (stepsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (stepsBuildStage == STAGE_UNINITIALIZED) {
        stepsBuildStage = STAGE_INITIALIZING;
        this.steps = createUnmodifiableList(false, createSafeList(getStepsInitialize(), true, false));
        stepsBuildStage = STAGE_INITIALIZED;
      }
      return this.steps;
    }

    void steps(List<Step> steps) {
      this.steps = steps;
      stepsBuildStage = STAGE_INITIALIZED;
    }
    private List<ResourceIdentifier> resources;
    private int resourcesBuildStage;

    List<ResourceIdentifier> getResources() {
      if (resourcesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (resourcesBuildStage == STAGE_UNINITIALIZED) {
        resourcesBuildStage = STAGE_INITIALIZING;
        this.resources = createUnmodifiableList(false, createSafeList(getResourcesInitialize(), true, false));
        resourcesBuildStage = STAGE_INITIALIZED;
      }
      return this.resources;
    }

    void resources(List<ResourceIdentifier> resources) {
      this.resources = resources;
      resourcesBuildStage = STAGE_INITIALIZED;
    }
    private int version;
    private int versionBuildStage;

    int getVersion() {
      if (versionBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (versionBuildStage == STAGE_UNINITIALIZED) {
        versionBuildStage = STAGE_INITIALIZING;
        this.version = getVersionInitialize();
        versionBuildStage = STAGE_INITIALIZED;
      }
      return this.version;
    }

    void version(int version) {
      this.version = version;
      versionBuildStage = STAGE_INITIALIZED;
    }
    private long createdAt;
    private int createdAtBuildStage;

    long getCreatedAt() {
      if (createdAtBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (createdAtBuildStage == STAGE_UNINITIALIZED) {
        createdAtBuildStage = STAGE_INITIALIZING;
        this.createdAt = getCreatedAtInitialize();
        createdAtBuildStage = STAGE_INITIALIZED;
      }
      return this.createdAt;
    }

    void createdAt(long createdAt) {
      this.createdAt = createdAt;
      createdAtBuildStage = STAGE_INITIALIZED;
    }
    private long updatedAt;
    private int updatedAtBuildStage;

    long getUpdatedAt() {
      if (updatedAtBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (updatedAtBuildStage == STAGE_UNINITIALIZED) {
        updatedAtBuildStage = STAGE_INITIALIZING;
        this.updatedAt = getUpdatedAtInitialize();
        updatedAtBuildStage = STAGE_INITIALIZED;
      }
      return this.updatedAt;
    }

    void updatedAt(long updatedAt) {
      this.updatedAt = updatedAt;
      updatedAtBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      ArrayList<String> attributes = new ArrayList<String>();
      if (isDeletedBuildStage == STAGE_INITIALIZING) attributes.add("isDeleted");
      if (connectionsBuildStage == STAGE_INITIALIZING) attributes.add("connections");
      if (stepsBuildStage == STAGE_INITIALIZING) attributes.add("steps");
      if (resourcesBuildStage == STAGE_INITIALIZING) attributes.add("resources");
      if (versionBuildStage == STAGE_INITIALIZING) attributes.add("version");
      if (createdAtBuildStage == STAGE_INITIALIZING) attributes.add("createdAt");
      if (updatedAtBuildStage == STAGE_INITIALIZING) attributes.add("updatedAt");
      return "Cannot build Integration, attribute initializers form cycle" + attributes;
    }
  }

  private boolean isDeletedInitialize() {
    return Integration.super.isDeleted();
  }

  private List<Connection> getConnectionsInitialize() {
    return Integration.super.getConnections();
  }

  private List<Step> getStepsInitialize() {
    return Integration.super.getSteps();
  }

  private List<ResourceIdentifier> getResourcesInitialize() {
    return Integration.super.getResources();
  }

  private int getVersionInitialize() {
    return Integration.super.getVersion();
  }

  private long getCreatedAtInitialize() {
    return Integration.super.getCreatedAt();
  }

  private long getUpdatedAtInitialize() {
    return Integration.super.getUpdatedAt();
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @JsonProperty("id")
  @Override
  public Optional<String> getId() {
    return Optional.ofNullable(id);
  }

  /**
   * @return The value of the {@code isDeleted} attribute
   */
  @JsonProperty("isDeleted")
  @JsonInclude(JsonInclude.Include.NON_DEFAULT)
  @Override
  public boolean isDeleted() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isDeleted()
        : this.isDeleted;
  }

  /**
   * @return The value of the {@code connections} attribute
   */
  @JsonProperty("connections")
  @Override
  public List<Connection> getConnections() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getConnections()
        : this.connections;
  }

  /**
   * @return The value of the {@code steps} attribute
   */
  @JsonProperty("steps")
  @Override
  public List<Step> getSteps() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getSteps()
        : this.steps;
  }

  /**
   * @return The value of the {@code resources} attribute
   */
  @JsonProperty("resources")
  @Override
  public List<ResourceIdentifier> getResources() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getResources()
        : this.resources;
  }

  /**
   * @return The value of the {@code scheduler} attribute
   */
  @JsonProperty("scheduler")
  @Override
  public Optional<Scheduler> getScheduler() {
    return Optional.ofNullable(scheduler);
  }

  /**
   * @return The value of the {@code description} attribute
   */
  @JsonProperty("description")
  @JsonDeserialize(converter = OptionalStringTrimmingConverter.class)
  @Override
  public Optional<String> getDescription() {
    return Optional.ofNullable(description);
  }

  /**
   * @return The value of the {@code version} attribute
   */
  @JsonProperty("version")
  @Override
  public int getVersion() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getVersion()
        : this.version;
  }

  /**
   * @return The value of the {@code createdAt} attribute
   */
  @JsonProperty("createdAt")
  @Override
  public long getCreatedAt() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getCreatedAt()
        : this.createdAt;
  }

  /**
   * @return The value of the {@code updatedAt} attribute
   */
  @JsonProperty("updatedAt")
  @Override
  public long getUpdatedAt() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getUpdatedAt()
        : this.updatedAt;
  }

  /**
   * @return The value of the {@code tags} attribute
   */
  @JsonProperty("tags")
  @JsonDeserialize(contentConverter = StringTrimmingConverter.class)
  @Override
  public SortedSet<String> getTags() {
    return tags;
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @JsonProperty("name")
  @JsonDeserialize(converter = StringTrimmingConverter.class)
  @Override
  public String getName() {
    return name;
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getId() id} attribute.
   * @param value The value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (Objects.equals(this.id, newValue)) return this;
    return validate(new ImmutableIntegration(
        this,
        newValue,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getId() id} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.id, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        value,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Integration#isDeleted() isDeleted} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for isDeleted
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegration withIsDeleted(boolean value) {
    if (this.isDeleted == value) return this;
    return validate(new ImmutableIntegration(
        this,
        this.id,
        value,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getConnections() connections}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withConnections(Connection... elements) {
    List<Connection> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        newValue,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getConnections() connections}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of connections elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withConnections(Iterable<? extends Connection> elements) {
    if (this.connections == elements) return this;
    List<Connection> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        newValue,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getSteps() steps}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withSteps(Step... elements) {
    List<Step> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        newValue,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getSteps() steps}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of steps elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withSteps(Iterable<? extends Step> elements) {
    if (this.steps == elements) return this;
    List<Step> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        newValue,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getResources() resources}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withResources(ResourceIdentifier... elements) {
    List<ResourceIdentifier> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        newValue,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getResources() resources}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of resources elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withResources(Iterable<? extends ResourceIdentifier> elements) {
    if (this.resources == elements) return this;
    List<ResourceIdentifier> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        newValue,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getScheduler() scheduler} attribute.
   * @param value The value for scheduler
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withScheduler(Scheduler value) {
    Scheduler newValue = Objects.requireNonNull(value, "scheduler");
    if (this.scheduler == newValue) return this;
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        newValue,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getScheduler() scheduler} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for scheduler
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withScheduler(Optional<? extends Scheduler> optional) {
    Scheduler value = optional.orElse(null);
    if (this.scheduler == value) return this;
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        value,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getDescription() description} attribute.
   * @param value The value for description
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withDescription(String value) {
    String newValue = Objects.requireNonNull(value, "description");
    if (Objects.equals(this.description, newValue)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        newValue,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getDescription() description} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for description
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withDescription(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.description, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        value,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Integration#getVersion() version} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for version
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegration withVersion(int value) {
    if (this.version == value) return this;
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        value,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Integration#getCreatedAt() createdAt} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for createdAt
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegration withCreatedAt(long value) {
    if (this.createdAt == value) return this;
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        value,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Integration#getUpdatedAt() updatedAt} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for updatedAt
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegration withUpdatedAt(long value) {
    if (this.updatedAt == value) return this;
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        value,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getTags() tags}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withTags(String... elements) {
    SortedSet<String> newValue = createUnmodifiableSortedSet(false, createSafeList(Arrays.asList(elements), false, true));
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        newValue,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getTags() tags}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of tags elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withTags(Iterable<String> elements) {
    if (this.tags == elements) return this;
    SortedSet<String> newValue = createUnmodifiableSortedSet(false, createSafeList(elements, false, true));
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        newValue,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Integration#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegration withName(String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableIntegration} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableIntegration
        && equalTo((ImmutableIntegration) another);
  }

  private boolean equalTo(ImmutableIntegration another) {
    return Objects.equals(id, another.id)
        && isDeleted == another.isDeleted
        && connections.equals(another.connections)
        && steps.equals(another.steps)
        && resources.equals(another.resources)
        && Objects.equals(scheduler, another.scheduler)
        && Objects.equals(description, another.description)
        && version == another.version
        && createdAt == another.createdAt
        && updatedAt == another.updatedAt
        && tags.equals(another.tags)
        && Objects.equals(name, another.name);
  }

  /**
   * Computes a hash code from attributes: {@code id}, {@code isDeleted}, {@code connections}, {@code steps}, {@code resources}, {@code scheduler}, {@code description}, {@code version}, {@code createdAt}, {@code updatedAt}, {@code tags}, {@code name}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(id);
    h += (h << 5) + Boolean.hashCode(isDeleted);
    h += (h << 5) + connections.hashCode();
    h += (h << 5) + steps.hashCode();
    h += (h << 5) + resources.hashCode();
    h += (h << 5) + Objects.hashCode(scheduler);
    h += (h << 5) + Objects.hashCode(description);
    h += (h << 5) + version;
    h += (h << 5) + Long.hashCode(createdAt);
    h += (h << 5) + Long.hashCode(updatedAt);
    h += (h << 5) + tags.hashCode();
    h += (h << 5) + Objects.hashCode(name);
    return h;
  }

  /**
   * Prints the immutable value {@code Integration} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("Integration{");
    if (id != null) {
      builder.append("id=").append(id);
    }
    if (builder.length() > 12) builder.append(", ");
    builder.append("isDeleted=").append(isDeleted);
    builder.append(", ");
    builder.append("connections=").append(connections);
    builder.append(", ");
    builder.append("steps=").append(steps);
    builder.append(", ");
    builder.append("resources=").append(resources);
    if (scheduler != null) {
      builder.append(", ");
      builder.append("scheduler=").append(scheduler);
    }
    if (description != null) {
      builder.append(", ");
      builder.append("description=").append(description);
    }
    builder.append(", ");
    builder.append("version=").append(version);
    builder.append(", ");
    builder.append("createdAt=").append(createdAt);
    builder.append(", ");
    builder.append("updatedAt=").append(updatedAt);
    builder.append(", ");
    builder.append("tags=").append(tags);
    if (name != null) {
      builder.append(", ");
      builder.append("name=").append(name);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code Integration} instance.
   * @param id The value for the {@code id} attribute
   * @param isDeleted The value for the {@code isDeleted} attribute
   * @param connections The value for the {@code connections} attribute
   * @param steps The value for the {@code steps} attribute
   * @param resources The value for the {@code resources} attribute
   * @param scheduler The value for the {@code scheduler} attribute
   * @param description The value for the {@code description} attribute
   * @param version The value for the {@code version} attribute
   * @param createdAt The value for the {@code createdAt} attribute
   * @param updatedAt The value for the {@code updatedAt} attribute
   * @param tags The value for the {@code tags} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable Integration instance
   */
  public static Integration of(Optional<String> id, boolean isDeleted, List<Connection> connections, List<Step> steps, List<ResourceIdentifier> resources, Optional<Scheduler> scheduler, Optional<String> description, int version, long createdAt, long updatedAt, SortedSet<String> tags, String name) {
    return of(id, isDeleted, (Iterable<? extends Connection>) connections, (Iterable<? extends Step>) steps, (Iterable<? extends ResourceIdentifier>) resources, scheduler, description, version, createdAt, updatedAt, (Iterable<String>) tags, name);
  }

  /**
   * Construct a new immutable {@code Integration} instance.
   * @param id The value for the {@code id} attribute
   * @param isDeleted The value for the {@code isDeleted} attribute
   * @param connections The value for the {@code connections} attribute
   * @param steps The value for the {@code steps} attribute
   * @param resources The value for the {@code resources} attribute
   * @param scheduler The value for the {@code scheduler} attribute
   * @param description The value for the {@code description} attribute
   * @param version The value for the {@code version} attribute
   * @param createdAt The value for the {@code createdAt} attribute
   * @param updatedAt The value for the {@code updatedAt} attribute
   * @param tags The value for the {@code tags} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable Integration instance
   */
  public static Integration of(Optional<String> id, boolean isDeleted, Iterable<? extends Connection> connections, Iterable<? extends Step> steps, Iterable<? extends ResourceIdentifier> resources, Optional<Scheduler> scheduler, Optional<String> description, int version, long createdAt, long updatedAt, Iterable<String> tags, String name) {
    return validate(new ImmutableIntegration(id, isDeleted, connections, steps, resources, scheduler, description, version, createdAt, updatedAt, tags, name));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableIntegration validate(ImmutableIntegration instance) {
    Set<ConstraintViolation<ImmutableIntegration>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link Integration} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Integration instance
   */
  public static Integration copyOf(Integration instance) {
    if (instance instanceof ImmutableIntegration) {
      return (ImmutableIntegration) instance;
    }
    return new Integration.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link Integration Integration}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  public static class Builder {
    private static final long OPT_BIT_IS_DELETED = 0x1L;
    private static final long OPT_BIT_CONNECTIONS = 0x2L;
    private static final long OPT_BIT_STEPS = 0x4L;
    private static final long OPT_BIT_RESOURCES = 0x8L;
    private static final long OPT_BIT_VERSION = 0x10L;
    private static final long OPT_BIT_CREATED_AT = 0x20L;
    private static final long OPT_BIT_UPDATED_AT = 0x40L;
    private long optBits;

    private String id;
    private boolean isDeleted;
    private List<Connection> connections = new ArrayList<Connection>();
    private List<Step> steps = new ArrayList<Step>();
    private List<ResourceIdentifier> resources = new ArrayList<ResourceIdentifier>();
    private Scheduler scheduler;
    private String description;
    private int version;
    private long createdAt;
    private long updatedAt;
    private List<String> tags = new ArrayList<String>();
    private String name;

    /**
     * Creates a builder for {@link Integration Integration} instances.
     */
    public Builder() {
      if (!(this instanceof Integration.Builder)) {
        throw new UnsupportedOperationException("Use: new Integration.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithName} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder createFrom(WithName instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Integration.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.integration.IntegrationBase} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder createFrom(IntegrationBase instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Integration.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithVersion} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder createFrom(WithVersion instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Integration.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithResourceId} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder createFrom(WithResourceId instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Integration.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithTags} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder createFrom(WithTags instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Integration.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithModificationTimestamps} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder createFrom(WithModificationTimestamps instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Integration.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.integration.Integration} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder createFrom(Integration instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Integration.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof WithName) {
        WithName instance = (WithName) object;
        String nameValue = instance.getName();
        if (nameValue != null) {
          name(nameValue);
        }
      }
      if (object instanceof IntegrationBase) {
        IntegrationBase instance = (IntegrationBase) object;
        Optional<Scheduler> schedulerOptional = instance.getScheduler();
        if (schedulerOptional.isPresent()) {
          scheduler(schedulerOptional);
        }
        addAllResources(instance.getResources());
        Optional<String> descriptionOptional = instance.getDescription();
        if (descriptionOptional.isPresent()) {
          description(descriptionOptional);
        }
        isDeleted(instance.isDeleted());
        addAllSteps(instance.getSteps());
        addAllConnections(instance.getConnections());
      }
      if (object instanceof WithVersion) {
        WithVersion instance = (WithVersion) object;
        version(instance.getVersion());
      }
      if (object instanceof WithResourceId) {
        WithResourceId instance = (WithResourceId) object;
        Optional<String> idOptional = instance.getId();
        if (idOptional.isPresent()) {
          id(idOptional);
        }
      }
      if (object instanceof WithTags) {
        WithTags instance = (WithTags) object;
        addAllTags(instance.getTags());
      }
      if (object instanceof WithModificationTimestamps) {
        WithModificationTimestamps instance = (WithModificationTimestamps) object;
        createdAt(instance.getCreatedAt());
        updatedAt(instance.getUpdatedAt());
      }
    }

    /**
     * Initializes the optional value {@link Integration#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder id(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("id")
    public final Integration.Builder id(Optional<String> id) {
      this.id = id.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the value for the {@link Integration#isDeleted() isDeleted} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link Integration#isDeleted() isDeleted}.</em>
     * @param isDeleted The value for isDeleted 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("isDeleted")
    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    public final Integration.Builder isDeleted(boolean isDeleted) {
      this.isDeleted = isDeleted;
      optBits |= OPT_BIT_IS_DELETED;
      return (Integration.Builder) this;
    }

    /**
     * Adds one element to {@link Integration#getConnections() connections} list.
     * @param element A connections element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addConnection(Connection element) {
      this.connections.add(Objects.requireNonNull(element, "connections element"));
      optBits |= OPT_BIT_CONNECTIONS;
      return (Integration.Builder) this;
    }

    /**
     * Adds elements to {@link Integration#getConnections() connections} list.
     * @param elements An array of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addConnection(Connection... elements) {
      for (Connection element : elements) {
        this.connections.add(Objects.requireNonNull(element, "connections element"));
      }
      optBits |= OPT_BIT_CONNECTIONS;
      return (Integration.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Integration#getConnections() connections} list.
     * @param elements An iterable of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connections")
    public final Integration.Builder connections(Iterable<? extends Connection> elements) {
      this.connections.clear();
      return addAllConnections(elements);
    }

    /**
     * Adds elements to {@link Integration#getConnections() connections} list.
     * @param elements An iterable of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addAllConnections(Iterable<? extends Connection> elements) {
      for (Connection element : elements) {
        this.connections.add(Objects.requireNonNull(element, "connections element"));
      }
      optBits |= OPT_BIT_CONNECTIONS;
      return (Integration.Builder) this;
    }

    /**
     * Adds one element to {@link Integration#getSteps() steps} list.
     * @param element A steps element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addStep(Step element) {
      this.steps.add(Objects.requireNonNull(element, "steps element"));
      optBits |= OPT_BIT_STEPS;
      return (Integration.Builder) this;
    }

    /**
     * Adds elements to {@link Integration#getSteps() steps} list.
     * @param elements An array of steps elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addStep(Step... elements) {
      for (Step element : elements) {
        this.steps.add(Objects.requireNonNull(element, "steps element"));
      }
      optBits |= OPT_BIT_STEPS;
      return (Integration.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Integration#getSteps() steps} list.
     * @param elements An iterable of steps elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("steps")
    public final Integration.Builder steps(Iterable<? extends Step> elements) {
      this.steps.clear();
      return addAllSteps(elements);
    }

    /**
     * Adds elements to {@link Integration#getSteps() steps} list.
     * @param elements An iterable of steps elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addAllSteps(Iterable<? extends Step> elements) {
      for (Step element : elements) {
        this.steps.add(Objects.requireNonNull(element, "steps element"));
      }
      optBits |= OPT_BIT_STEPS;
      return (Integration.Builder) this;
    }

    /**
     * Adds one element to {@link Integration#getResources() resources} list.
     * @param element A resources element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addResource(ResourceIdentifier element) {
      this.resources.add(Objects.requireNonNull(element, "resources element"));
      optBits |= OPT_BIT_RESOURCES;
      return (Integration.Builder) this;
    }

    /**
     * Adds elements to {@link Integration#getResources() resources} list.
     * @param elements An array of resources elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addResource(ResourceIdentifier... elements) {
      for (ResourceIdentifier element : elements) {
        this.resources.add(Objects.requireNonNull(element, "resources element"));
      }
      optBits |= OPT_BIT_RESOURCES;
      return (Integration.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Integration#getResources() resources} list.
     * @param elements An iterable of resources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("resources")
    public final Integration.Builder resources(Iterable<? extends ResourceIdentifier> elements) {
      this.resources.clear();
      return addAllResources(elements);
    }

    /**
     * Adds elements to {@link Integration#getResources() resources} list.
     * @param elements An iterable of resources elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addAllResources(Iterable<? extends ResourceIdentifier> elements) {
      for (ResourceIdentifier element : elements) {
        this.resources.add(Objects.requireNonNull(element, "resources element"));
      }
      optBits |= OPT_BIT_RESOURCES;
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getScheduler() scheduler} to scheduler.
     * @param scheduler The value for scheduler
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder scheduler(Scheduler scheduler) {
      this.scheduler = Objects.requireNonNull(scheduler, "scheduler");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getScheduler() scheduler} to scheduler.
     * @param scheduler The value for scheduler
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("scheduler")
    public final Integration.Builder scheduler(Optional<? extends Scheduler> scheduler) {
      this.scheduler = scheduler.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getDescription() description} to description.
     * @param description The value for description
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder description(String description) {
      this.description = Objects.requireNonNull(description, "description");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getDescription() description} to description.
     * @param description The value for description
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("description")
    @JsonDeserialize(converter = OptionalStringTrimmingConverter.class)
    public final Integration.Builder description(Optional<String> description) {
      this.description = description.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the value for the {@link Integration#getVersion() version} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link Integration#getVersion() version}.</em>
     * @param version The value for version 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("version")
    public final Integration.Builder version(int version) {
      this.version = version;
      optBits |= OPT_BIT_VERSION;
      return (Integration.Builder) this;
    }

    /**
     * Initializes the value for the {@link Integration#getCreatedAt() createdAt} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link Integration#getCreatedAt() createdAt}.</em>
     * @param createdAt The value for createdAt 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("createdAt")
    public final Integration.Builder createdAt(long createdAt) {
      this.createdAt = createdAt;
      optBits |= OPT_BIT_CREATED_AT;
      return (Integration.Builder) this;
    }

    /**
     * Initializes the value for the {@link Integration#getUpdatedAt() updatedAt} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link Integration#getUpdatedAt() updatedAt}.</em>
     * @param updatedAt The value for updatedAt 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("updatedAt")
    public final Integration.Builder updatedAt(long updatedAt) {
      this.updatedAt = updatedAt;
      optBits |= OPT_BIT_UPDATED_AT;
      return (Integration.Builder) this;
    }

    /**
     * Adds one element to {@link Integration#getTags() tags} sortedSet.
     * @param element A tags element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addTag(String element) {
      if (element != null) this.tags.add(element);
      return (Integration.Builder) this;
    }

    /**
     * Adds elements to {@link Integration#getTags() tags} sortedSet.
     * @param elements An array of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addTag(String... elements) {
      for (String element : elements) {
        if (element != null) this.tags.add(element);
      }
      return (Integration.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Integration#getTags() tags} sortedSet.
     * @param elements An iterable of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("tags")
    @JsonDeserialize(contentConverter = StringTrimmingConverter.class)
    public final Integration.Builder tags(Iterable<String> elements) {
      this.tags.clear();
      return addAllTags(elements);
    }

    /**
     * Adds elements to {@link Integration#getTags() tags} sortedSet.
     * @param elements An iterable of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addAllTags(Iterable<String> elements) {
      for (String element : elements) {
        if (element != null) this.tags.add(element);
      }
      return (Integration.Builder) this;
    }

    /**
     * Initializes the value for the {@link Integration#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("name")
    @JsonDeserialize(converter = StringTrimmingConverter.class)
    public final Integration.Builder name(String name) {
      this.name = name;
      return (Integration.Builder) this;
    }

    /**
     * Builds a new {@link Integration Integration}.
     * @return An immutable instance of Integration
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public Integration build() {
      return ImmutableIntegration.validate(new ImmutableIntegration(this));
    }

    private boolean isDeletedIsSet() {
      return (optBits & OPT_BIT_IS_DELETED) != 0;
    }

    private boolean connectionsIsSet() {
      return (optBits & OPT_BIT_CONNECTIONS) != 0;
    }

    private boolean stepsIsSet() {
      return (optBits & OPT_BIT_STEPS) != 0;
    }

    private boolean resourcesIsSet() {
      return (optBits & OPT_BIT_RESOURCES) != 0;
    }

    private boolean versionIsSet() {
      return (optBits & OPT_BIT_VERSION) != 0;
    }

    private boolean createdAtIsSet() {
      return (optBits & OPT_BIT_CREATED_AT) != 0;
    }

    private boolean updatedAtIsSet() {
      return (optBits & OPT_BIT_UPDATED_AT) != 0;
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<T>();
    } else {
      list = new ArrayList<T>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T> List<T> createUnmodifiableList(boolean clone, List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptyList();
    case 1: return Collections.singletonList(list.get(0));
    default:
      if (clone) {
        return Collections.unmodifiableList(new ArrayList<T>(list));
      } else {
        if (list instanceof ArrayList<?>) {
          ((ArrayList<?>) list).trimToSize();
        }
        return Collections.unmodifiableList(list);
      }
    }
  }

  private static <T extends Comparable<T>> NavigableSet<T> createUnmodifiableSortedSet(boolean reverse, List<T> list) {
    TreeSet<T> set = reverse
        ? new TreeSet<T>(Collections.reverseOrder())
        : new TreeSet<T>();
    set.addAll(list);
    return Collections.unmodifiableNavigableSet(set);
  }
}
