/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.common.model.integration;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.common.model.ResourceIdentifier;
import io.syndesis.common.model.WithModificationTimestamps;
import io.syndesis.common.model.WithName;
import io.syndesis.common.model.WithResourceId;
import io.syndesis.common.model.WithTags;
import io.syndesis.common.model.WithVersion;
import io.syndesis.common.model.bulletin.IntegrationBulletinBoard;
import io.syndesis.common.model.connection.Connection;
import io.syndesis.common.util.json.OptionalStringTrimmingConverter;
import io.syndesis.common.util.json.StringTrimmingConverter;
import java.io.ObjectStreamException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.NavigableSet;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link IntegrationOverview}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new IntegrationOverview.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableIntegrationOverview.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "IntegrationOverview"})
final class ImmutableIntegrationOverview
    implements IntegrationOverview {
  private final boolean isDraft;
  private final IntegrationDeploymentState currentState;
  private final IntegrationDeploymentState targetState;
  private final Integer deploymentVersion;
  private final List<IntegrationDeploymentOverview> deployments;
  private final IntegrationBulletinBoard board;
  private final String id;
  private final boolean isDeleted;
  private final List<Connection> connections;
  private final List<Step> steps;
  private final List<ResourceIdentifier> resources;
  private final Scheduler scheduler;
  private final String description;
  private final int version;
  private final long createdAt;
  private final long updatedAt;
  private final SortedSet<String> tags;
  private final String name;

  private ImmutableIntegrationOverview(
      boolean isDraft,
      IntegrationDeploymentState currentState,
      IntegrationDeploymentState targetState,
      Optional<Integer> deploymentVersion,
      Iterable<? extends IntegrationDeploymentOverview> deployments,
      IntegrationBulletinBoard board,
      Optional<String> id,
      boolean isDeleted,
      Iterable<? extends Connection> connections,
      Iterable<? extends Step> steps,
      Iterable<? extends ResourceIdentifier> resources,
      Optional<Scheduler> scheduler,
      Optional<String> description,
      int version,
      long createdAt,
      long updatedAt,
      Iterable<String> tags,
      String name) {
    this.isDraft = isDraft;
    this.currentState = Objects.requireNonNull(currentState, "currentState");
    this.targetState = Objects.requireNonNull(targetState, "targetState");
    this.deploymentVersion = deploymentVersion.orElse(null);
    this.deployments = createUnmodifiableList(false, createSafeList(deployments, true, false));
    this.board = Objects.requireNonNull(board, "board");
    this.id = id.orElse(null);
    this.isDeleted = isDeleted;
    this.connections = createUnmodifiableList(false, createSafeList(connections, true, false));
    this.steps = createUnmodifiableList(false, createSafeList(steps, true, false));
    this.resources = createUnmodifiableList(false, createSafeList(resources, true, false));
    this.scheduler = scheduler.orElse(null);
    this.description = description.orElse(null);
    this.version = version;
    this.createdAt = createdAt;
    this.updatedAt = updatedAt;
    this.tags = createUnmodifiableSortedSet(false, createSafeList(tags, false, true));
    this.name = name;
    initShim.currentState(this.currentState);
    initShim.targetState(this.targetState);
    initShim.deployments(this.deployments);
    initShim.board(this.board);
    initShim.isDeleted(this.isDeleted);
    initShim.connections(this.connections);
    initShim.steps(this.steps);
    initShim.resources(this.resources);
    initShim.version(this.version);
    initShim.createdAt(this.createdAt);
    initShim.updatedAt(this.updatedAt);
    this.initShim = null;
  }

  private ImmutableIntegrationOverview(ImmutableIntegrationOverview.Builder builder) {
    this.isDraft = builder.isDraft;
    this.deploymentVersion = builder.deploymentVersion;
    this.id = builder.id;
    this.scheduler = builder.scheduler;
    this.description = builder.description;
    this.tags = createUnmodifiableSortedSet(false, createSafeList(builder.tags, false, false));
    this.name = builder.name;
    if (builder.currentState != null) {
      initShim.currentState(builder.currentState);
    }
    if (builder.targetState != null) {
      initShim.targetState(builder.targetState);
    }
    if (builder.deploymentsIsSet()) {
      initShim.deployments(createUnmodifiableList(true, builder.deployments));
    }
    if (builder.board != null) {
      initShim.board(builder.board);
    }
    if (builder.isDeletedIsSet()) {
      initShim.isDeleted(builder.isDeleted);
    }
    if (builder.connectionsIsSet()) {
      initShim.connections(createUnmodifiableList(true, builder.connections));
    }
    if (builder.stepsIsSet()) {
      initShim.steps(createUnmodifiableList(true, builder.steps));
    }
    if (builder.resourcesIsSet()) {
      initShim.resources(createUnmodifiableList(true, builder.resources));
    }
    if (builder.versionIsSet()) {
      initShim.version(builder.version);
    }
    if (builder.createdAtIsSet()) {
      initShim.createdAt(builder.createdAt);
    }
    if (builder.updatedAtIsSet()) {
      initShim.updatedAt(builder.updatedAt);
    }
    this.currentState = initShim.getCurrentState();
    this.targetState = initShim.getTargetState();
    this.deployments = initShim.getDeployments();
    this.board = initShim.getBoard();
    this.isDeleted = initShim.isDeleted();
    this.connections = initShim.getConnections();
    this.steps = initShim.getSteps();
    this.resources = initShim.getResources();
    this.version = initShim.getVersion();
    this.createdAt = initShim.getCreatedAt();
    this.updatedAt = initShim.getUpdatedAt();
    this.initShim = null;
  }

  private ImmutableIntegrationOverview(
      ImmutableIntegrationOverview original,
      boolean isDraft,
      IntegrationDeploymentState currentState,
      IntegrationDeploymentState targetState,
      Integer deploymentVersion,
      List<IntegrationDeploymentOverview> deployments,
      IntegrationBulletinBoard board,
      String id,
      boolean isDeleted,
      List<Connection> connections,
      List<Step> steps,
      List<ResourceIdentifier> resources,
      Scheduler scheduler,
      String description,
      int version,
      long createdAt,
      long updatedAt,
      SortedSet<String> tags,
      String name) {
    this.isDraft = isDraft;
    this.currentState = currentState;
    this.targetState = targetState;
    this.deploymentVersion = deploymentVersion;
    this.deployments = deployments;
    this.board = board;
    this.id = id;
    this.isDeleted = isDeleted;
    this.connections = connections;
    this.steps = steps;
    this.resources = resources;
    this.scheduler = scheduler;
    this.description = description;
    this.version = version;
    this.createdAt = createdAt;
    this.updatedAt = updatedAt;
    this.tags = tags;
    this.name = name;
    this.initShim = null;
  }

  private static final int STAGE_INITIALIZING = -1;
  private static final int STAGE_UNINITIALIZED = 0;
  private static final int STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  private final class InitShim {
    private IntegrationDeploymentState currentState;
    private int currentStateBuildStage;

    IntegrationDeploymentState getCurrentState() {
      if (currentStateBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (currentStateBuildStage == STAGE_UNINITIALIZED) {
        currentStateBuildStage = STAGE_INITIALIZING;
        this.currentState = Objects.requireNonNull(getCurrentStateInitialize(), "currentState");
        currentStateBuildStage = STAGE_INITIALIZED;
      }
      return this.currentState;
    }

    void currentState(IntegrationDeploymentState currentState) {
      this.currentState = currentState;
      currentStateBuildStage = STAGE_INITIALIZED;
    }
    private IntegrationDeploymentState targetState;
    private int targetStateBuildStage;

    IntegrationDeploymentState getTargetState() {
      if (targetStateBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (targetStateBuildStage == STAGE_UNINITIALIZED) {
        targetStateBuildStage = STAGE_INITIALIZING;
        this.targetState = Objects.requireNonNull(getTargetStateInitialize(), "targetState");
        targetStateBuildStage = STAGE_INITIALIZED;
      }
      return this.targetState;
    }

    void targetState(IntegrationDeploymentState targetState) {
      this.targetState = targetState;
      targetStateBuildStage = STAGE_INITIALIZED;
    }
    private List<IntegrationDeploymentOverview> deployments;
    private int deploymentsBuildStage;

    List<IntegrationDeploymentOverview> getDeployments() {
      if (deploymentsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (deploymentsBuildStage == STAGE_UNINITIALIZED) {
        deploymentsBuildStage = STAGE_INITIALIZING;
        this.deployments = createUnmodifiableList(false, createSafeList(getDeploymentsInitialize(), true, false));
        deploymentsBuildStage = STAGE_INITIALIZED;
      }
      return this.deployments;
    }

    void deployments(List<IntegrationDeploymentOverview> deployments) {
      this.deployments = deployments;
      deploymentsBuildStage = STAGE_INITIALIZED;
    }
    private IntegrationBulletinBoard board;
    private int boardBuildStage;

    IntegrationBulletinBoard getBoard() {
      if (boardBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (boardBuildStage == STAGE_UNINITIALIZED) {
        boardBuildStage = STAGE_INITIALIZING;
        this.board = Objects.requireNonNull(getBoardInitialize(), "board");
        boardBuildStage = STAGE_INITIALIZED;
      }
      return this.board;
    }

    void board(IntegrationBulletinBoard board) {
      this.board = board;
      boardBuildStage = STAGE_INITIALIZED;
    }
    private boolean isDeleted;
    private int isDeletedBuildStage;

    boolean isDeleted() {
      if (isDeletedBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (isDeletedBuildStage == STAGE_UNINITIALIZED) {
        isDeletedBuildStage = STAGE_INITIALIZING;
        this.isDeleted = isDeletedInitialize();
        isDeletedBuildStage = STAGE_INITIALIZED;
      }
      return this.isDeleted;
    }

    void isDeleted(boolean isDeleted) {
      this.isDeleted = isDeleted;
      isDeletedBuildStage = STAGE_INITIALIZED;
    }
    private List<Connection> connections;
    private int connectionsBuildStage;

    List<Connection> getConnections() {
      if (connectionsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (connectionsBuildStage == STAGE_UNINITIALIZED) {
        connectionsBuildStage = STAGE_INITIALIZING;
        this.connections = createUnmodifiableList(false, createSafeList(getConnectionsInitialize(), true, false));
        connectionsBuildStage = STAGE_INITIALIZED;
      }
      return this.connections;
    }

    void connections(List<Connection> connections) {
      this.connections = connections;
      connectionsBuildStage = STAGE_INITIALIZED;
    }
    private List<Step> steps;
    private int stepsBuildStage;

    List<Step> getSteps() {
      if (stepsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (stepsBuildStage == STAGE_UNINITIALIZED) {
        stepsBuildStage = STAGE_INITIALIZING;
        this.steps = createUnmodifiableList(false, createSafeList(getStepsInitialize(), true, false));
        stepsBuildStage = STAGE_INITIALIZED;
      }
      return this.steps;
    }

    void steps(List<Step> steps) {
      this.steps = steps;
      stepsBuildStage = STAGE_INITIALIZED;
    }
    private List<ResourceIdentifier> resources;
    private int resourcesBuildStage;

    List<ResourceIdentifier> getResources() {
      if (resourcesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (resourcesBuildStage == STAGE_UNINITIALIZED) {
        resourcesBuildStage = STAGE_INITIALIZING;
        this.resources = createUnmodifiableList(false, createSafeList(getResourcesInitialize(), true, false));
        resourcesBuildStage = STAGE_INITIALIZED;
      }
      return this.resources;
    }

    void resources(List<ResourceIdentifier> resources) {
      this.resources = resources;
      resourcesBuildStage = STAGE_INITIALIZED;
    }
    private int version;
    private int versionBuildStage;

    int getVersion() {
      if (versionBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (versionBuildStage == STAGE_UNINITIALIZED) {
        versionBuildStage = STAGE_INITIALIZING;
        this.version = getVersionInitialize();
        versionBuildStage = STAGE_INITIALIZED;
      }
      return this.version;
    }

    void version(int version) {
      this.version = version;
      versionBuildStage = STAGE_INITIALIZED;
    }
    private long createdAt;
    private int createdAtBuildStage;

    long getCreatedAt() {
      if (createdAtBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (createdAtBuildStage == STAGE_UNINITIALIZED) {
        createdAtBuildStage = STAGE_INITIALIZING;
        this.createdAt = getCreatedAtInitialize();
        createdAtBuildStage = STAGE_INITIALIZED;
      }
      return this.createdAt;
    }

    void createdAt(long createdAt) {
      this.createdAt = createdAt;
      createdAtBuildStage = STAGE_INITIALIZED;
    }
    private long updatedAt;
    private int updatedAtBuildStage;

    long getUpdatedAt() {
      if (updatedAtBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (updatedAtBuildStage == STAGE_UNINITIALIZED) {
        updatedAtBuildStage = STAGE_INITIALIZING;
        this.updatedAt = getUpdatedAtInitialize();
        updatedAtBuildStage = STAGE_INITIALIZED;
      }
      return this.updatedAt;
    }

    void updatedAt(long updatedAt) {
      this.updatedAt = updatedAt;
      updatedAtBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      ArrayList<String> attributes = new ArrayList<String>();
      if (currentStateBuildStage == STAGE_INITIALIZING) attributes.add("currentState");
      if (targetStateBuildStage == STAGE_INITIALIZING) attributes.add("targetState");
      if (deploymentsBuildStage == STAGE_INITIALIZING) attributes.add("deployments");
      if (boardBuildStage == STAGE_INITIALIZING) attributes.add("board");
      if (isDeletedBuildStage == STAGE_INITIALIZING) attributes.add("isDeleted");
      if (connectionsBuildStage == STAGE_INITIALIZING) attributes.add("connections");
      if (stepsBuildStage == STAGE_INITIALIZING) attributes.add("steps");
      if (resourcesBuildStage == STAGE_INITIALIZING) attributes.add("resources");
      if (versionBuildStage == STAGE_INITIALIZING) attributes.add("version");
      if (createdAtBuildStage == STAGE_INITIALIZING) attributes.add("createdAt");
      if (updatedAtBuildStage == STAGE_INITIALIZING) attributes.add("updatedAt");
      return "Cannot build IntegrationOverview, attribute initializers form cycle" + attributes;
    }
  }

  private IntegrationDeploymentState getCurrentStateInitialize() {
    return IntegrationOverview.super.getCurrentState();
  }

  private IntegrationDeploymentState getTargetStateInitialize() {
    return IntegrationOverview.super.getTargetState();
  }

  private List<IntegrationDeploymentOverview> getDeploymentsInitialize() {
    return IntegrationOverview.super.getDeployments();
  }

  private IntegrationBulletinBoard getBoardInitialize() {
    return IntegrationOverview.super.getBoard();
  }

  private boolean isDeletedInitialize() {
    return IntegrationOverview.super.isDeleted();
  }

  private List<Connection> getConnectionsInitialize() {
    return IntegrationOverview.super.getConnections();
  }

  private List<Step> getStepsInitialize() {
    return IntegrationOverview.super.getSteps();
  }

  private List<ResourceIdentifier> getResourcesInitialize() {
    return IntegrationOverview.super.getResources();
  }

  private int getVersionInitialize() {
    return IntegrationOverview.super.getVersion();
  }

  private long getCreatedAtInitialize() {
    return IntegrationOverview.super.getCreatedAt();
  }

  private long getUpdatedAtInitialize() {
    return IntegrationOverview.super.getUpdatedAt();
  }

  /**
   * @return The value of the {@code isDraft} attribute
   */
  @JsonProperty("isDraft")
  @Override
  public boolean isDraft() {
    return isDraft;
  }

  /**
   * @return The value of the {@code currentState} attribute
   */
  @JsonProperty("currentState")
  @Override
  public IntegrationDeploymentState getCurrentState() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getCurrentState()
        : this.currentState;
  }

  /**
   * @return The value of the {@code targetState} attribute
   */
  @JsonProperty("targetState")
  @Override
  public IntegrationDeploymentState getTargetState() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getTargetState()
        : this.targetState;
  }

  /**
   * @return The value of the {@code deploymentVersion} attribute
   */
  @JsonProperty("deploymentVersion")
  @Override
  public Optional<Integer> getDeploymentVersion() {
    return Optional.ofNullable(deploymentVersion);
  }

  /**
   * @return The value of the {@code deployments} attribute
   */
  @JsonProperty("deployments")
  @Override
  public List<IntegrationDeploymentOverview> getDeployments() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getDeployments()
        : this.deployments;
  }

  /**
   * @return The value of the {@code board} attribute
   */
  @JsonProperty("board")
  @Override
  public IntegrationBulletinBoard getBoard() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getBoard()
        : this.board;
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @JsonProperty("id")
  @Override
  public Optional<String> getId() {
    return Optional.ofNullable(id);
  }

  /**
   * @return The value of the {@code isDeleted} attribute
   */
  @JsonProperty("isDeleted")
  @JsonInclude(JsonInclude.Include.NON_DEFAULT)
  @Override
  public boolean isDeleted() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isDeleted()
        : this.isDeleted;
  }

  /**
   * @return The value of the {@code connections} attribute
   */
  @JsonProperty("connections")
  @Override
  public List<Connection> getConnections() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getConnections()
        : this.connections;
  }

  /**
   * @return The value of the {@code steps} attribute
   */
  @JsonProperty("steps")
  @Override
  public List<Step> getSteps() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getSteps()
        : this.steps;
  }

  /**
   * @return The value of the {@code resources} attribute
   */
  @JsonProperty("resources")
  @Override
  public List<ResourceIdentifier> getResources() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getResources()
        : this.resources;
  }

  /**
   * @return The value of the {@code scheduler} attribute
   */
  @JsonProperty("scheduler")
  @Override
  public Optional<Scheduler> getScheduler() {
    return Optional.ofNullable(scheduler);
  }

  /**
   * @return The value of the {@code description} attribute
   */
  @JsonProperty("description")
  @JsonDeserialize(converter = OptionalStringTrimmingConverter.class)
  @Override
  public Optional<String> getDescription() {
    return Optional.ofNullable(description);
  }

  /**
   * @return The value of the {@code version} attribute
   */
  @JsonProperty("version")
  @Override
  public int getVersion() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getVersion()
        : this.version;
  }

  /**
   * @return The value of the {@code createdAt} attribute
   */
  @JsonProperty("createdAt")
  @Override
  public long getCreatedAt() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getCreatedAt()
        : this.createdAt;
  }

  /**
   * @return The value of the {@code updatedAt} attribute
   */
  @JsonProperty("updatedAt")
  @Override
  public long getUpdatedAt() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getUpdatedAt()
        : this.updatedAt;
  }

  /**
   * @return The value of the {@code tags} attribute
   */
  @JsonProperty("tags")
  @JsonDeserialize(contentConverter = StringTrimmingConverter.class)
  @Override
  public SortedSet<String> getTags() {
    return tags;
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @JsonProperty("name")
  @JsonDeserialize(converter = StringTrimmingConverter.class)
  @Override
  public String getName() {
    return name;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationOverview#isDraft() isDraft} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for isDraft
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationOverview withIsDraft(boolean value) {
    if (this.isDraft == value) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        value,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationOverview#getCurrentState() currentState} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for currentState
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationOverview withCurrentState(IntegrationDeploymentState value) {
    if (this.currentState == value) return this;
    IntegrationDeploymentState newValue = Objects.requireNonNull(value, "currentState");
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        newValue,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationOverview#getTargetState() targetState} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for targetState
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationOverview withTargetState(IntegrationDeploymentState value) {
    if (this.targetState == value) return this;
    IntegrationDeploymentState newValue = Objects.requireNonNull(value, "targetState");
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        newValue,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationOverview#getDeploymentVersion() deploymentVersion} attribute.
   * @param value The value for deploymentVersion
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withDeploymentVersion(int value) {
    Integer newValue = value;
    if (Objects.equals(this.deploymentVersion, newValue)) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        newValue,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationOverview#getDeploymentVersion() deploymentVersion} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for deploymentVersion
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withDeploymentVersion(Optional<Integer> optional) {
    Integer value = optional.orElse(null);
    if (Objects.equals(this.deploymentVersion, value)) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        value,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationOverview#getDeployments() deployments}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withDeployments(IntegrationDeploymentOverview... elements) {
    List<IntegrationDeploymentOverview> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        newValue,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationOverview#getDeployments() deployments}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of deployments elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withDeployments(Iterable<? extends IntegrationDeploymentOverview> elements) {
    if (this.deployments == elements) return this;
    List<IntegrationDeploymentOverview> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        newValue,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationOverview#getBoard() board} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for board
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationOverview withBoard(IntegrationBulletinBoard value) {
    if (this.board == value) return this;
    IntegrationBulletinBoard newValue = Objects.requireNonNull(value, "board");
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        newValue,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationOverview#getId() id} attribute.
   * @param value The value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (Objects.equals(this.id, newValue)) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        newValue,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationOverview#getId() id} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.id, value)) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        value,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationOverview#isDeleted() isDeleted} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for isDeleted
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationOverview withIsDeleted(boolean value) {
    if (this.isDeleted == value) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        value,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationOverview#getConnections() connections}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withConnections(Connection... elements) {
    List<Connection> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        newValue,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationOverview#getConnections() connections}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of connections elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withConnections(Iterable<? extends Connection> elements) {
    if (this.connections == elements) return this;
    List<Connection> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        newValue,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationOverview#getSteps() steps}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withSteps(Step... elements) {
    List<Step> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        newValue,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationOverview#getSteps() steps}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of steps elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withSteps(Iterable<? extends Step> elements) {
    if (this.steps == elements) return this;
    List<Step> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        newValue,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationOverview#getResources() resources}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withResources(ResourceIdentifier... elements) {
    List<ResourceIdentifier> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        newValue,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationOverview#getResources() resources}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of resources elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withResources(Iterable<? extends ResourceIdentifier> elements) {
    if (this.resources == elements) return this;
    List<ResourceIdentifier> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        newValue,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationOverview#getScheduler() scheduler} attribute.
   * @param value The value for scheduler
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withScheduler(Scheduler value) {
    Scheduler newValue = Objects.requireNonNull(value, "scheduler");
    if (this.scheduler == newValue) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        newValue,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationOverview#getScheduler() scheduler} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for scheduler
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withScheduler(Optional<? extends Scheduler> optional) {
    Scheduler value = optional.orElse(null);
    if (this.scheduler == value) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        value,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationOverview#getDescription() description} attribute.
   * @param value The value for description
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withDescription(String value) {
    String newValue = Objects.requireNonNull(value, "description");
    if (Objects.equals(this.description, newValue)) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        newValue,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationOverview#getDescription() description} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for description
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withDescription(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.description, value)) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        value,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationOverview#getVersion() version} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for version
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationOverview withVersion(int value) {
    if (this.version == value) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        value,
        this.createdAt,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationOverview#getCreatedAt() createdAt} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for createdAt
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationOverview withCreatedAt(long value) {
    if (this.createdAt == value) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        value,
        this.updatedAt,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationOverview#getUpdatedAt() updatedAt} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for updatedAt
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationOverview withUpdatedAt(long value) {
    if (this.updatedAt == value) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        value,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationOverview#getTags() tags}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withTags(String... elements) {
    SortedSet<String> newValue = createUnmodifiableSortedSet(false, createSafeList(Arrays.asList(elements), false, true));
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        newValue,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationOverview#getTags() tags}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of tags elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationOverview withTags(Iterable<String> elements) {
    if (this.tags == elements) return this;
    SortedSet<String> newValue = createUnmodifiableSortedSet(false, createSafeList(elements, false, true));
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        newValue,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationOverview#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationOverview withName(String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableIntegrationOverview(
        this,
        this.isDraft,
        this.currentState,
        this.targetState,
        this.deploymentVersion,
        this.deployments,
        this.board,
        this.id,
        this.isDeleted,
        this.connections,
        this.steps,
        this.resources,
        this.scheduler,
        this.description,
        this.version,
        this.createdAt,
        this.updatedAt,
        this.tags,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableIntegrationOverview} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableIntegrationOverview
        && equalTo((ImmutableIntegrationOverview) another);
  }

  private boolean equalTo(ImmutableIntegrationOverview another) {
    return isDraft == another.isDraft
        && currentState.equals(another.currentState)
        && targetState.equals(another.targetState)
        && Objects.equals(deploymentVersion, another.deploymentVersion)
        && deployments.equals(another.deployments)
        && board.equals(another.board)
        && Objects.equals(id, another.id)
        && isDeleted == another.isDeleted
        && connections.equals(another.connections)
        && steps.equals(another.steps)
        && resources.equals(another.resources)
        && Objects.equals(scheduler, another.scheduler)
        && Objects.equals(description, another.description)
        && version == another.version
        && createdAt == another.createdAt
        && updatedAt == another.updatedAt
        && tags.equals(another.tags)
        && Objects.equals(name, another.name);
  }

  /**
   * Computes a hash code from attributes: {@code isDraft}, {@code currentState}, {@code targetState}, {@code deploymentVersion}, {@code deployments}, {@code board}, {@code id}, {@code isDeleted}, {@code connections}, {@code steps}, {@code resources}, {@code scheduler}, {@code description}, {@code version}, {@code createdAt}, {@code updatedAt}, {@code tags}, {@code name}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Boolean.hashCode(isDraft);
    h += (h << 5) + currentState.hashCode();
    h += (h << 5) + targetState.hashCode();
    h += (h << 5) + Objects.hashCode(deploymentVersion);
    h += (h << 5) + deployments.hashCode();
    h += (h << 5) + board.hashCode();
    h += (h << 5) + Objects.hashCode(id);
    h += (h << 5) + Boolean.hashCode(isDeleted);
    h += (h << 5) + connections.hashCode();
    h += (h << 5) + steps.hashCode();
    h += (h << 5) + resources.hashCode();
    h += (h << 5) + Objects.hashCode(scheduler);
    h += (h << 5) + Objects.hashCode(description);
    h += (h << 5) + version;
    h += (h << 5) + Long.hashCode(createdAt);
    h += (h << 5) + Long.hashCode(updatedAt);
    h += (h << 5) + tags.hashCode();
    h += (h << 5) + Objects.hashCode(name);
    return h;
  }

  /**
   * Prints the immutable value {@code IntegrationOverview} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("IntegrationOverview{");
    builder.append("isDraft=").append(isDraft);
    builder.append(", ");
    builder.append("currentState=").append(currentState);
    builder.append(", ");
    builder.append("targetState=").append(targetState);
    if (deploymentVersion != null) {
      builder.append(", ");
      builder.append("deploymentVersion=").append(deploymentVersion);
    }
    builder.append(", ");
    builder.append("deployments=").append(deployments);
    builder.append(", ");
    builder.append("board=").append(board);
    if (id != null) {
      builder.append(", ");
      builder.append("id=").append(id);
    }
    builder.append(", ");
    builder.append("isDeleted=").append(isDeleted);
    builder.append(", ");
    builder.append("connections=").append(connections);
    builder.append(", ");
    builder.append("steps=").append(steps);
    builder.append(", ");
    builder.append("resources=").append(resources);
    if (scheduler != null) {
      builder.append(", ");
      builder.append("scheduler=").append(scheduler);
    }
    if (description != null) {
      builder.append(", ");
      builder.append("description=").append(description);
    }
    builder.append(", ");
    builder.append("version=").append(version);
    builder.append(", ");
    builder.append("createdAt=").append(createdAt);
    builder.append(", ");
    builder.append("updatedAt=").append(updatedAt);
    builder.append(", ");
    builder.append("tags=").append(tags);
    if (name != null) {
      builder.append(", ");
      builder.append("name=").append(name);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code IntegrationOverview} instance.
   * @param isDraft The value for the {@code isDraft} attribute
   * @param currentState The value for the {@code currentState} attribute
   * @param targetState The value for the {@code targetState} attribute
   * @param deploymentVersion The value for the {@code deploymentVersion} attribute
   * @param deployments The value for the {@code deployments} attribute
   * @param board The value for the {@code board} attribute
   * @param id The value for the {@code id} attribute
   * @param isDeleted The value for the {@code isDeleted} attribute
   * @param connections The value for the {@code connections} attribute
   * @param steps The value for the {@code steps} attribute
   * @param resources The value for the {@code resources} attribute
   * @param scheduler The value for the {@code scheduler} attribute
   * @param description The value for the {@code description} attribute
   * @param version The value for the {@code version} attribute
   * @param createdAt The value for the {@code createdAt} attribute
   * @param updatedAt The value for the {@code updatedAt} attribute
   * @param tags The value for the {@code tags} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable IntegrationOverview instance
   */
  public static IntegrationOverview of(boolean isDraft, IntegrationDeploymentState currentState, IntegrationDeploymentState targetState, Optional<Integer> deploymentVersion, List<IntegrationDeploymentOverview> deployments, IntegrationBulletinBoard board, Optional<String> id, boolean isDeleted, List<Connection> connections, List<Step> steps, List<ResourceIdentifier> resources, Optional<Scheduler> scheduler, Optional<String> description, int version, long createdAt, long updatedAt, SortedSet<String> tags, String name) {
    return of(isDraft, currentState, targetState, deploymentVersion, (Iterable<? extends IntegrationDeploymentOverview>) deployments, board, id, isDeleted, (Iterable<? extends Connection>) connections, (Iterable<? extends Step>) steps, (Iterable<? extends ResourceIdentifier>) resources, scheduler, description, version, createdAt, updatedAt, (Iterable<String>) tags, name);
  }

  /**
   * Construct a new immutable {@code IntegrationOverview} instance.
   * @param isDraft The value for the {@code isDraft} attribute
   * @param currentState The value for the {@code currentState} attribute
   * @param targetState The value for the {@code targetState} attribute
   * @param deploymentVersion The value for the {@code deploymentVersion} attribute
   * @param deployments The value for the {@code deployments} attribute
   * @param board The value for the {@code board} attribute
   * @param id The value for the {@code id} attribute
   * @param isDeleted The value for the {@code isDeleted} attribute
   * @param connections The value for the {@code connections} attribute
   * @param steps The value for the {@code steps} attribute
   * @param resources The value for the {@code resources} attribute
   * @param scheduler The value for the {@code scheduler} attribute
   * @param description The value for the {@code description} attribute
   * @param version The value for the {@code version} attribute
   * @param createdAt The value for the {@code createdAt} attribute
   * @param updatedAt The value for the {@code updatedAt} attribute
   * @param tags The value for the {@code tags} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable IntegrationOverview instance
   */
  public static IntegrationOverview of(boolean isDraft, IntegrationDeploymentState currentState, IntegrationDeploymentState targetState, Optional<Integer> deploymentVersion, Iterable<? extends IntegrationDeploymentOverview> deployments, IntegrationBulletinBoard board, Optional<String> id, boolean isDeleted, Iterable<? extends Connection> connections, Iterable<? extends Step> steps, Iterable<? extends ResourceIdentifier> resources, Optional<Scheduler> scheduler, Optional<String> description, int version, long createdAt, long updatedAt, Iterable<String> tags, String name) {
    return validate(new ImmutableIntegrationOverview(isDraft, currentState, targetState, deploymentVersion, deployments, board, id, isDeleted, connections, steps, resources, scheduler, description, version, createdAt, updatedAt, tags, name));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableIntegrationOverview validate(ImmutableIntegrationOverview instance) {
    Set<ConstraintViolation<ImmutableIntegrationOverview>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link IntegrationOverview} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable IntegrationOverview instance
   */
  public static IntegrationOverview copyOf(IntegrationOverview instance) {
    if (instance instanceof ImmutableIntegrationOverview) {
      return (ImmutableIntegrationOverview) instance;
    }
    return new IntegrationOverview.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link IntegrationOverview IntegrationOverview}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  public static class Builder {
    private static final long OPT_BIT_DEPLOYMENTS = 0x1L;
    private static final long OPT_BIT_IS_DELETED = 0x2L;
    private static final long OPT_BIT_CONNECTIONS = 0x4L;
    private static final long OPT_BIT_STEPS = 0x8L;
    private static final long OPT_BIT_RESOURCES = 0x10L;
    private static final long OPT_BIT_VERSION = 0x20L;
    private static final long OPT_BIT_CREATED_AT = 0x40L;
    private static final long OPT_BIT_UPDATED_AT = 0x80L;
    private long optBits;

    private boolean isDraft;
    private IntegrationDeploymentState currentState;
    private IntegrationDeploymentState targetState;
    private Integer deploymentVersion;
    private List<IntegrationDeploymentOverview> deployments = new ArrayList<IntegrationDeploymentOverview>();
    private IntegrationBulletinBoard board;
    private String id;
    private boolean isDeleted;
    private List<Connection> connections = new ArrayList<Connection>();
    private List<Step> steps = new ArrayList<Step>();
    private List<ResourceIdentifier> resources = new ArrayList<ResourceIdentifier>();
    private Scheduler scheduler;
    private String description;
    private int version;
    private long createdAt;
    private long updatedAt;
    private List<String> tags = new ArrayList<String>();
    private String name;

    /**
     * Creates a builder for {@link IntegrationOverview IntegrationOverview} instances.
     */
    public Builder() {
      if (!(this instanceof IntegrationOverview.Builder)) {
        throw new UnsupportedOperationException("Use: new IntegrationOverview.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithName} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder createFrom(WithName instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.integration.IntegrationBase} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder createFrom(IntegrationBase instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithVersion} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder createFrom(WithVersion instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.integration.IntegrationOverview} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder createFrom(IntegrationOverview instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithTags} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder createFrom(WithTags instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithResourceId} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder createFrom(WithResourceId instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithModificationTimestamps} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder createFrom(WithModificationTimestamps instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (IntegrationOverview.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof WithName) {
        WithName instance = (WithName) object;
        String nameValue = instance.getName();
        if (nameValue != null) {
          name(nameValue);
        }
      }
      if (object instanceof IntegrationBase) {
        IntegrationBase instance = (IntegrationBase) object;
        Optional<Scheduler> schedulerOptional = instance.getScheduler();
        if (schedulerOptional.isPresent()) {
          scheduler(schedulerOptional);
        }
        addAllResources(instance.getResources());
        Optional<String> descriptionOptional = instance.getDescription();
        if (descriptionOptional.isPresent()) {
          description(descriptionOptional);
        }
        isDeleted(instance.isDeleted());
        addAllSteps(instance.getSteps());
        addAllConnections(instance.getConnections());
      }
      if (object instanceof WithVersion) {
        WithVersion instance = (WithVersion) object;
        version(instance.getVersion());
      }
      if (object instanceof IntegrationOverview) {
        IntegrationOverview instance = (IntegrationOverview) object;
        isDraft(instance.isDraft());
        addAllDeployments(instance.getDeployments());
        currentState(instance.getCurrentState());
        Optional<Integer> deploymentVersionOptional = instance.getDeploymentVersion();
        if (deploymentVersionOptional.isPresent()) {
          deploymentVersion(deploymentVersionOptional);
        }
        targetState(instance.getTargetState());
        board(instance.getBoard());
      }
      if (object instanceof WithTags) {
        WithTags instance = (WithTags) object;
        addAllTags(instance.getTags());
      }
      if (object instanceof WithResourceId) {
        WithResourceId instance = (WithResourceId) object;
        Optional<String> idOptional = instance.getId();
        if (idOptional.isPresent()) {
          id(idOptional);
        }
      }
      if (object instanceof WithModificationTimestamps) {
        WithModificationTimestamps instance = (WithModificationTimestamps) object;
        createdAt(instance.getCreatedAt());
        updatedAt(instance.getUpdatedAt());
      }
    }

    /**
     * Initializes the value for the {@link IntegrationOverview#isDraft() isDraft} attribute.
     * @param isDraft The value for isDraft 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("isDraft")
    public final IntegrationOverview.Builder isDraft(boolean isDraft) {
      this.isDraft = isDraft;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationOverview#getCurrentState() currentState} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IntegrationOverview#getCurrentState() currentState}.</em>
     * @param currentState The value for currentState 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("currentState")
    public final IntegrationOverview.Builder currentState(IntegrationDeploymentState currentState) {
      this.currentState = Objects.requireNonNull(currentState, "currentState");
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationOverview#getTargetState() targetState} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IntegrationOverview#getTargetState() targetState}.</em>
     * @param targetState The value for targetState 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("targetState")
    public final IntegrationOverview.Builder targetState(IntegrationDeploymentState targetState) {
      this.targetState = Objects.requireNonNull(targetState, "targetState");
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationOverview#getDeploymentVersion() deploymentVersion} to deploymentVersion.
     * @param deploymentVersion The value for deploymentVersion
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationOverview.Builder deploymentVersion(int deploymentVersion) {
      this.deploymentVersion = deploymentVersion;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationOverview#getDeploymentVersion() deploymentVersion} to deploymentVersion.
     * @param deploymentVersion The value for deploymentVersion
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("deploymentVersion")
    public final IntegrationOverview.Builder deploymentVersion(Optional<Integer> deploymentVersion) {
      this.deploymentVersion = deploymentVersion.orElse(null);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationOverview#getDeployments() deployments} list.
     * @param element A deployments element
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addDeployment(IntegrationDeploymentOverview element) {
      this.deployments.add(Objects.requireNonNull(element, "deployments element"));
      optBits |= OPT_BIT_DEPLOYMENTS;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationOverview#getDeployments() deployments} list.
     * @param elements An array of deployments elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addDeployment(IntegrationDeploymentOverview... elements) {
      for (IntegrationDeploymentOverview element : elements) {
        this.deployments.add(Objects.requireNonNull(element, "deployments element"));
      }
      optBits |= OPT_BIT_DEPLOYMENTS;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link IntegrationOverview#getDeployments() deployments} list.
     * @param elements An iterable of deployments elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("deployments")
    public final IntegrationOverview.Builder deployments(Iterable<? extends IntegrationDeploymentOverview> elements) {
      this.deployments.clear();
      return addAllDeployments(elements);
    }

    /**
     * Adds elements to {@link IntegrationOverview#getDeployments() deployments} list.
     * @param elements An iterable of deployments elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addAllDeployments(Iterable<? extends IntegrationDeploymentOverview> elements) {
      for (IntegrationDeploymentOverview element : elements) {
        this.deployments.add(Objects.requireNonNull(element, "deployments element"));
      }
      optBits |= OPT_BIT_DEPLOYMENTS;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationOverview#getBoard() board} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IntegrationOverview#getBoard() board}.</em>
     * @param board The value for board 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("board")
    public final IntegrationOverview.Builder board(IntegrationBulletinBoard board) {
      this.board = Objects.requireNonNull(board, "board");
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationOverview#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationOverview.Builder id(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationOverview#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("id")
    public final IntegrationOverview.Builder id(Optional<String> id) {
      this.id = id.orElse(null);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationOverview#isDeleted() isDeleted} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IntegrationOverview#isDeleted() isDeleted}.</em>
     * @param isDeleted The value for isDeleted 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("isDeleted")
    @JsonInclude(JsonInclude.Include.NON_DEFAULT)
    public final IntegrationOverview.Builder isDeleted(boolean isDeleted) {
      this.isDeleted = isDeleted;
      optBits |= OPT_BIT_IS_DELETED;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationOverview#getConnections() connections} list.
     * @param element A connections element
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addConnection(Connection element) {
      this.connections.add(Objects.requireNonNull(element, "connections element"));
      optBits |= OPT_BIT_CONNECTIONS;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationOverview#getConnections() connections} list.
     * @param elements An array of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addConnection(Connection... elements) {
      for (Connection element : elements) {
        this.connections.add(Objects.requireNonNull(element, "connections element"));
      }
      optBits |= OPT_BIT_CONNECTIONS;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link IntegrationOverview#getConnections() connections} list.
     * @param elements An iterable of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connections")
    public final IntegrationOverview.Builder connections(Iterable<? extends Connection> elements) {
      this.connections.clear();
      return addAllConnections(elements);
    }

    /**
     * Adds elements to {@link IntegrationOverview#getConnections() connections} list.
     * @param elements An iterable of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addAllConnections(Iterable<? extends Connection> elements) {
      for (Connection element : elements) {
        this.connections.add(Objects.requireNonNull(element, "connections element"));
      }
      optBits |= OPT_BIT_CONNECTIONS;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationOverview#getSteps() steps} list.
     * @param element A steps element
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addStep(Step element) {
      this.steps.add(Objects.requireNonNull(element, "steps element"));
      optBits |= OPT_BIT_STEPS;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationOverview#getSteps() steps} list.
     * @param elements An array of steps elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addStep(Step... elements) {
      for (Step element : elements) {
        this.steps.add(Objects.requireNonNull(element, "steps element"));
      }
      optBits |= OPT_BIT_STEPS;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link IntegrationOverview#getSteps() steps} list.
     * @param elements An iterable of steps elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("steps")
    public final IntegrationOverview.Builder steps(Iterable<? extends Step> elements) {
      this.steps.clear();
      return addAllSteps(elements);
    }

    /**
     * Adds elements to {@link IntegrationOverview#getSteps() steps} list.
     * @param elements An iterable of steps elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addAllSteps(Iterable<? extends Step> elements) {
      for (Step element : elements) {
        this.steps.add(Objects.requireNonNull(element, "steps element"));
      }
      optBits |= OPT_BIT_STEPS;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationOverview#getResources() resources} list.
     * @param element A resources element
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addResource(ResourceIdentifier element) {
      this.resources.add(Objects.requireNonNull(element, "resources element"));
      optBits |= OPT_BIT_RESOURCES;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationOverview#getResources() resources} list.
     * @param elements An array of resources elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addResource(ResourceIdentifier... elements) {
      for (ResourceIdentifier element : elements) {
        this.resources.add(Objects.requireNonNull(element, "resources element"));
      }
      optBits |= OPT_BIT_RESOURCES;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link IntegrationOverview#getResources() resources} list.
     * @param elements An iterable of resources elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("resources")
    public final IntegrationOverview.Builder resources(Iterable<? extends ResourceIdentifier> elements) {
      this.resources.clear();
      return addAllResources(elements);
    }

    /**
     * Adds elements to {@link IntegrationOverview#getResources() resources} list.
     * @param elements An iterable of resources elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addAllResources(Iterable<? extends ResourceIdentifier> elements) {
      for (ResourceIdentifier element : elements) {
        this.resources.add(Objects.requireNonNull(element, "resources element"));
      }
      optBits |= OPT_BIT_RESOURCES;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationOverview#getScheduler() scheduler} to scheduler.
     * @param scheduler The value for scheduler
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationOverview.Builder scheduler(Scheduler scheduler) {
      this.scheduler = Objects.requireNonNull(scheduler, "scheduler");
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationOverview#getScheduler() scheduler} to scheduler.
     * @param scheduler The value for scheduler
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("scheduler")
    public final IntegrationOverview.Builder scheduler(Optional<? extends Scheduler> scheduler) {
      this.scheduler = scheduler.orElse(null);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationOverview#getDescription() description} to description.
     * @param description The value for description
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationOverview.Builder description(String description) {
      this.description = Objects.requireNonNull(description, "description");
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationOverview#getDescription() description} to description.
     * @param description The value for description
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("description")
    @JsonDeserialize(converter = OptionalStringTrimmingConverter.class)
    public final IntegrationOverview.Builder description(Optional<String> description) {
      this.description = description.orElse(null);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationOverview#getVersion() version} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IntegrationOverview#getVersion() version}.</em>
     * @param version The value for version 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("version")
    public final IntegrationOverview.Builder version(int version) {
      this.version = version;
      optBits |= OPT_BIT_VERSION;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationOverview#getCreatedAt() createdAt} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IntegrationOverview#getCreatedAt() createdAt}.</em>
     * @param createdAt The value for createdAt 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("createdAt")
    public final IntegrationOverview.Builder createdAt(long createdAt) {
      this.createdAt = createdAt;
      optBits |= OPT_BIT_CREATED_AT;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationOverview#getUpdatedAt() updatedAt} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IntegrationOverview#getUpdatedAt() updatedAt}.</em>
     * @param updatedAt The value for updatedAt 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("updatedAt")
    public final IntegrationOverview.Builder updatedAt(long updatedAt) {
      this.updatedAt = updatedAt;
      optBits |= OPT_BIT_UPDATED_AT;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationOverview#getTags() tags} sortedSet.
     * @param element A tags element
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addTag(String element) {
      if (element != null) this.tags.add(element);
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationOverview#getTags() tags} sortedSet.
     * @param elements An array of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addTag(String... elements) {
      for (String element : elements) {
        if (element != null) this.tags.add(element);
      }
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link IntegrationOverview#getTags() tags} sortedSet.
     * @param elements An iterable of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("tags")
    @JsonDeserialize(contentConverter = StringTrimmingConverter.class)
    public final IntegrationOverview.Builder tags(Iterable<String> elements) {
      this.tags.clear();
      return addAllTags(elements);
    }

    /**
     * Adds elements to {@link IntegrationOverview#getTags() tags} sortedSet.
     * @param elements An iterable of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationOverview.Builder addAllTags(Iterable<String> elements) {
      for (String element : elements) {
        if (element != null) this.tags.add(element);
      }
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationOverview#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("name")
    @JsonDeserialize(converter = StringTrimmingConverter.class)
    public final IntegrationOverview.Builder name(String name) {
      this.name = name;
      return (IntegrationOverview.Builder) this;
    }

    /**
     * Builds a new {@link IntegrationOverview IntegrationOverview}.
     * @return An immutable instance of IntegrationOverview
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public IntegrationOverview build() {
      return ImmutableIntegrationOverview.validate(new ImmutableIntegrationOverview(this));
    }

    private boolean deploymentsIsSet() {
      return (optBits & OPT_BIT_DEPLOYMENTS) != 0;
    }

    private boolean isDeletedIsSet() {
      return (optBits & OPT_BIT_IS_DELETED) != 0;
    }

    private boolean connectionsIsSet() {
      return (optBits & OPT_BIT_CONNECTIONS) != 0;
    }

    private boolean stepsIsSet() {
      return (optBits & OPT_BIT_STEPS) != 0;
    }

    private boolean resourcesIsSet() {
      return (optBits & OPT_BIT_RESOURCES) != 0;
    }

    private boolean versionIsSet() {
      return (optBits & OPT_BIT_VERSION) != 0;
    }

    private boolean createdAtIsSet() {
      return (optBits & OPT_BIT_CREATED_AT) != 0;
    }

    private boolean updatedAtIsSet() {
      return (optBits & OPT_BIT_UPDATED_AT) != 0;
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<T>();
    } else {
      list = new ArrayList<T>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T> List<T> createUnmodifiableList(boolean clone, List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptyList();
    case 1: return Collections.singletonList(list.get(0));
    default:
      if (clone) {
        return Collections.unmodifiableList(new ArrayList<T>(list));
      } else {
        if (list instanceof ArrayList<?>) {
          ((ArrayList<?>) list).trimToSize();
        }
        return Collections.unmodifiableList(list);
      }
    }
  }

  private static <T extends Comparable<T>> NavigableSet<T> createUnmodifiableSortedSet(boolean reverse, List<T> list) {
    TreeSet<T> set = reverse
        ? new TreeSet<T>(Collections.reverseOrder())
        : new TreeSet<T>();
    set.addAll(list);
    return Collections.unmodifiableNavigableSet(set);
  }
}
